@@ -1,6 +1,4 @@
 /*
- * $Id: adfile.h 6215 2012-03-20 13:30:51Z matthijs $
- *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,8 +33,7 @@
 #define ADAPTER_ADFILE_H
 
 #include "config.h"
-#include "shared/allocator.h"
-#include "shared/status.h"
+#include "status.h"
 
 #include <stdio.h>
 
