@@ -33,20 +33,20 @@
 #include "daemon/cfg.h"
 #include "daemon/engine.h"
 #include "daemon/signal.h"
-#include "shared/allocator.h"
-#include "shared/duration.h"
-#include "shared/file.h"
-#include "shared/hsm.h"
-#include "shared/locks.h"
-#include "shared/log.h"
-#include "shared/privdrop.h"
-#include "shared/status.h"
-#include "shared/util.h"
+#include "duration.h"
+#include "file.h"
+#include "str.h"
+#include "hsm.h"
+#include "locks.h"
+#include "log.h"
+#include "privdrop.h"
+#include "status.h"
+#include "util.h"
 #include "signer/zonelist.h"
 #include "wire/tsig.h"
+#include "libhsm.h"
 
 #include <errno.h>
-#include <libhsm.h>
 #include <libxml/parser.h>
 #include <signal.h>
 #include <stdio.h>
@@ -70,20 +70,7 @@ static engine_type*
 engine_create(void)
 {
     engine_type* engine;
-    allocator_type* allocator = allocator_create(malloc, free);
-    if (!allocator) {
-        ods_log_error("[%s] unable to create engine: allocator_create() "
-            "failed", engine_str);
-        return NULL;
-    }
-    engine = (engine_type*) allocator_alloc(allocator, sizeof(engine_type));
-    if (!engine) {
-        ods_log_error("[%s] unable to create engine: allocator_alloc() "
-            "failed", engine_str);
-        allocator_cleanup(allocator);
-        return NULL;
-    }
-    engine->allocator = allocator;
+    CHECKALLOC(engine = (engine_type*) malloc(sizeof(engine_type)));
     engine->config = NULL;
     engine->workers = NULL;
     engine->drudgers = NULL;
@@ -91,6 +78,8 @@ engine_create(void)
     engine->cmdhandler_done = 0;
     engine->dnshandler = NULL;
     engine->xfrhandler = NULL;
+    engine->taskq = NULL;
+    engine->signq = NULL;
     engine->pid = -1;
     engine->uid = -1;
     engine->gid = -1;
@@ -102,17 +91,17 @@ engine_create(void)
     lock_basic_lock(&engine->signal_lock);
     engine->signal = SIGNAL_INIT;
     lock_basic_unlock(&engine->signal_lock);
-    engine->zonelist = zonelist_create(engine->allocator);
+    engine->zonelist = zonelist_create();
     if (!engine->zonelist) {
         engine_cleanup(engine);
         return NULL;
     }
-    engine->taskq = schedule_create(engine->allocator);
+    engine->taskq = schedule_create();
     if (!engine->taskq) {
         engine_cleanup(engine);
         return NULL;
     }
-    engine->signq = fifoq_create(engine->allocator);
+    engine->signq = fifoq_create();
     if (!engine->signq) {
         engine_cleanup(engine);
         return NULL;
@@ -125,6 +114,7 @@ engine_create(void)
  * Start command handler.
  *
  */
+
 static void*
 cmdhandler_thread_start(void* arg)
 {
@@ -133,6 +123,7 @@ cmdhandler_thread_start(void* arg)
     cmdhandler_start(cmd);
     return NULL;
 }
+
 static void
 engine_start_cmdhandler(engine_type* engine)
 {
@@ -141,8 +132,8 @@ engine_start_cmdhandler(engine_type* engine)
     engine->cmdhandler->engine = engine;
     ods_thread_create(&engine->cmdhandler->thread_id,
         cmdhandler_thread_start, engine->cmdhandler);
-    return;
 }
+
 /**
  * Self pipe trick (see Unix Network Programming).
  *
@@ -163,8 +154,7 @@ self_pipe_trick(engine_type* engine)
     } else {
         bzero(&servaddr, sizeof(servaddr));
         servaddr.sun_family = AF_UNIX;
-        strncpy(servaddr.sun_path, servsock_filename,
-            sizeof(servaddr.sun_path) - 1);
+        strncpy(servaddr.sun_path, servsock_filename, sizeof(servaddr.sun_path)-1);
         ret = connect(sockfd, (const struct sockaddr*) &servaddr,
             sizeof(servaddr));
         if (ret != 0) {
@@ -227,7 +217,6 @@ engine_start_dnshandler(engine_type* engine)
     engine->dnshandler->engine = engine;
     ods_thread_create(&engine->dnshandler->thread_id,
         dnshandler_thread_start, engine->dnshandler);
-    return;
 }
 static void
 engine_stop_dnshandler(engine_type* engine)
@@ -241,7 +230,6 @@ engine_stop_dnshandler(engine_type* engine)
     ods_log_debug("[%s] join dnshandler", engine_str);
     ods_thread_join(engine->dnshandler->thread_id);
     engine->dnshandler->engine = NULL;
-    return;
 }
 
 
@@ -264,14 +252,13 @@ engine_start_xfrhandler(engine_type* engine)
     }
     ods_log_debug("[%s] start xfrhandler", engine_str);
     engine->xfrhandler->engine = engine;
-    ods_thread_create(&engine->xfrhandler->thread_id,
-        xfrhandler_thread_start, engine->xfrhandler);
     /* This might be the wrong place to mark the xfrhandler started but
      * if its isn't done here we might try to shutdown and stop it before
      * it has marked itself started
      */
     engine->xfrhandler->started = 1;
-    return;
+    ods_thread_create(&engine->xfrhandler->thread_id,
+        xfrhandler_thread_start, engine->xfrhandler);
 }
 static void
 engine_stop_xfrhandler(engine_type* engine)
@@ -288,7 +275,6 @@ engine_stop_xfrhandler(engine_type* engine)
     	engine->xfrhandler->started = 0;
     }
     engine->xfrhandler->engine = NULL;
-    return;
 }
 
 
@@ -338,14 +324,10 @@ engine_create_workers(engine_type* engine)
     size_t i = 0;
     ods_log_assert(engine);
     ods_log_assert(engine->config);
-    ods_log_assert(engine->allocator);
-    engine->workers = (worker_type**) allocator_alloc(engine->allocator,
-        ((size_t)engine->config->num_worker_threads) * sizeof(worker_type*));
+    CHECKALLOC(engine->workers = (worker_type**) malloc(((size_t)engine->config->num_worker_threads) * sizeof(worker_type*)));
     for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
-        engine->workers[i] = worker_create(engine->allocator, i,
-            WORKER_WORKER);
+        engine->workers[i] = worker_create(i, WORKER_WORKER);
     }
-    return;
 }
 static void
 engine_create_drudgers(engine_type* engine)
@@ -353,14 +335,10 @@ engine_create_drudgers(engine_type* engine)
     size_t i = 0;
     ods_log_assert(engine);
     ods_log_assert(engine->config);
-    ods_log_assert(engine->allocator);
-    engine->drudgers = (worker_type**) allocator_alloc(engine->allocator,
-        ((size_t)engine->config->num_signer_threads) * sizeof(worker_type*));
+    CHECKALLOC(engine->drudgers = (worker_type**) malloc(((size_t)engine->config->num_signer_threads) * sizeof(worker_type*)));
     for (i=0; i < (size_t) engine->config->num_signer_threads; i++) {
-        engine->drudgers[i] = worker_create(engine->allocator, i,
-            WORKER_DRUDGER);
+        engine->drudgers[i] = worker_create(i, WORKER_DRUDGER);
     }
-    return;
 }
 static void*
 worker_thread_start(void* arg)
@@ -383,7 +361,6 @@ engine_start_workers(engine_type* engine)
         ods_thread_create(&engine->workers[i]->thread_id, worker_thread_start,
             engine->workers[i]);
     }
-    return;
 }
 void
 engine_start_drudgers(engine_type* engine)
@@ -398,50 +375,47 @@ engine_start_drudgers(engine_type* engine)
         ods_thread_create(&engine->drudgers[i]->thread_id, worker_thread_start,
             engine->drudgers[i]);
     }
-    return;
 }
 static void
 engine_stop_workers(engine_type* engine)
 {
-    size_t i = 0;
+    int i = 0;
     ods_log_assert(engine);
     ods_log_assert(engine->config);
     ods_log_debug("[%s] stop workers", engine_str);
     /* tell them to exit and wake up sleepyheads */
-    for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
+    for (i=0; i < engine->config->num_worker_threads; i++) {
         engine->workers[i]->need_to_exit = 1;
         worker_wakeup(engine->workers[i]);
     }
     ods_log_debug("[%s] notify workers", engine_str);
     worker_notify_all(&engine->signq->q_lock, &engine->signq->q_nonfull);
     /* head count */
-    for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
-        ods_log_debug("[%s] join worker %i", engine_str, i+1);
+    for (i=0; i < engine->config->num_worker_threads; i++) {
+        ods_log_debug("[%s] join worker %d", engine_str, i+1);
         ods_thread_join(engine->workers[i]->thread_id);
         engine->workers[i]->engine = NULL;
     }
-    return;
 }
 void
 engine_stop_drudgers(engine_type* engine)
 {
-    size_t i = 0;
+    int i = 0;
     ods_log_assert(engine);
     ods_log_assert(engine->config);
     ods_log_debug("[%s] stop drudgers", engine_str);
     /* tell them to exit and wake up sleepyheads */
-    for (i=0; i < (size_t) engine->config->num_signer_threads; i++) {
+    for (i=0; i < engine->config->num_signer_threads; i++) {
         engine->drudgers[i]->need_to_exit = 1;
     }
     ods_log_debug("[%s] notify drudgers", engine_str);
     worker_notify_all(&engine->signq->q_lock, &engine->signq->q_threshold);
     /* head count */
-    for (i=0; i < (size_t) engine->config->num_signer_threads; i++) {
-        ods_log_debug("[%s] join drudger %i", engine_str, i+1);
+    for (i=0; i < engine->config->num_signer_threads; i++) {
+        ods_log_debug("[%s] join drudger %d", engine_str, i+1);
         ods_thread_join(engine->drudgers[i]->thread_id);
         engine->drudgers[i]->engine = NULL;
     }
-    return;
 }
 
 
@@ -460,7 +434,6 @@ engine_wakeup_workers(engine_type* engine)
     for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
         worker_wakeup(engine->workers[i]);
     }
-    return;
 }
 
 
@@ -473,7 +446,6 @@ engine_setup(engine_type* engine)
 {
     ods_status status = ODS_STATUS_OK;
     struct sigaction action;
-    int result = 0;
     int sockets[2] = {0,0};
 
     ods_log_debug("[%s] setup signer engine", engine_str);
@@ -484,14 +456,12 @@ engine_setup(engine_type* engine)
     edns_init(&engine->edns, EDNS_MAX_MESSAGE_LEN);
 
     /* create command handler (before chowning socket file) */
-    engine->cmdhandler = cmdhandler_create(engine->allocator,
-        engine->config->clisock_filename);
+    engine->cmdhandler = cmdhandler_create(engine->config->clisock_filename);
     if (!engine->cmdhandler) {
         return ODS_STATUS_CMDHANDLER_ERR;
     }
-    engine->dnshandler = dnshandler_create(engine->allocator,
-        engine->config->interfaces);
-    engine->xfrhandler = xfrhandler_create(engine->allocator);
+    engine->dnshandler = dnshandler_create(engine->config->interfaces);
+    engine->xfrhandler = xfrhandler_create();
     if (!engine->xfrhandler) {
         return ODS_STATUS_XFRHANDLER_ERR;
     }
@@ -505,6 +475,7 @@ engine_setup(engine_type* engine)
         if (status != ODS_STATUS_OK) {
             ods_log_error("[%s] setup: unable to listen to sockets (%s)",
                 engine_str, ods_status2str(status));
+            return ODS_STATUS_XFRHANDLER_ERR;
         }
     }
     /* privdrop */
@@ -551,6 +522,11 @@ engine_setup(engine_type* engine)
         }
     }
     engine->pid = getpid();
+    /* write pidfile */
+    if (util_write_pidfile(engine->config->pid_filename, engine->pid) == -1) {
+        return ODS_STATUS_WRITE_PIDFILE_ERR;
+    }
+    /* setup done */
     ods_log_verbose("[%s] running as pid %lu", engine_str,
         (unsigned long) engine->pid);
     /* catch signals */
@@ -567,11 +543,6 @@ engine_setup(engine_type* engine)
     sigaction(SIGCHLD, &action, NULL);
     action.sa_handler = SIG_IGN;
     sigaction(SIGPIPE, &action, NULL);
-    /* set up hsm */ /* LEAK */
-    result = lhsm_open(engine->config->cfg_filename);
-    if (result != HSM_OK) {
-        return ODS_STATUS_HSM_ERR;
-    }
     /* create workers/drudgers */
     engine_create_workers(engine);
     engine_create_drudgers(engine);
@@ -579,13 +550,7 @@ engine_setup(engine_type* engine)
     engine_start_cmdhandler(engine);
     engine_start_dnshandler(engine);
     engine_start_xfrhandler(engine);
-    tsig_handler_init(engine->allocator);
-    /* write pidfile */
-    if (util_write_pidfile(engine->config->pid_filename, engine->pid) == -1) {
-        hsm_close();
-        return ODS_STATUS_WRITE_PIDFILE_ERR;
-    }
-    /* setup done */
+    tsig_handler_init();
     return ODS_STATUS_OK;
 }
 
@@ -643,6 +608,7 @@ engine_run(engine_type* engine, int single_run)
                 ods_log_assert(1);
                 break;
             case SIGNAL_RELOAD:
+                ods_log_error("signer instructed to reload due to explicit signal");
                 engine->need_to_reload = 1;
                 break;
             case SIGNAL_SHUTDOWN:
@@ -650,7 +616,7 @@ engine_run(engine_type* engine, int single_run)
                 break;
             default:
                 ods_log_warning("[%s] invalid signal %d captured, "
-                    "keep running", engine_str, signal);
+                    "keep running", engine_str, (int)engine->signal);
                 engine->signal = SIGNAL_RUN;
                 break;
         }
@@ -669,8 +635,6 @@ engine_run(engine_type* engine, int single_run)
     ods_log_debug("[%s] signer halted", engine_str);
     engine_stop_drudgers(engine);
     engine_stop_workers(engine);
-    (void)lhsm_reopen(engine->config->cfg_filename);
-    return;
 }
 
 
@@ -700,7 +664,7 @@ set_notify_ns(zone_type* zone, const char* cmd)
         str2 = ods_replace(cmd, "%zone", zone->name);
     }
     if (str2) {
-        ods_str_trim((char*) str2);
+        ods_str_trim((char*) str2, 1);
         str = str2;
         if (*str) {
             token = NULL;
@@ -718,7 +682,6 @@ set_notify_ns(zone_type* zone, const char* cmd)
         ods_log_error("[%s] unable to set notify ns: replace zone failed",
             engine_str);
     }
-    return;
 }
 
 
@@ -898,7 +861,6 @@ engine_update_zones(engine_type* engine, ods_status zl_changed)
     if (wake_up) {
         engine_wakeup_workers(engine);
     }
-    return;
 }
 
 
@@ -979,34 +941,23 @@ engine_recover(engine_type* engine)
  * Start engine.
  *
  */
-void
+int
 engine_start(const char* cfgfile, int cmdline_verbosity, int daemonize,
     int info, int single_run)
 {
     engine_type* engine = NULL;
-    int use_syslog = 0;
     ods_status zl_changed = ODS_STATUS_UNCHANGED;
     ods_status status = ODS_STATUS_OK;
-    int close_hsm = 0;
 
-    ods_log_assert(cfgfile);
-    ods_log_init(NULL, use_syslog, cmdline_verbosity);
-    ods_log_verbose("[%s] starting signer", engine_str);
-
-    /* initialize */
-    xmlInitGlobals();
-    xmlInitParser();
-    xmlInitThreads();
     engine = engine_create();
     if (!engine) {
         ods_fatal_exit("[%s] create failed", engine_str);
-        return;
+        return 1;
     }
     engine->daemonize = daemonize;
 
     /* config */
-    engine->config = engine_config(engine->allocator, cfgfile,
-        cmdline_verbosity);
+    engine->config = engine_config(cfgfile, cmdline_verbosity);
     status = engine_config_check(engine->config);
     if (status != ODS_STATUS_OK) {
         ods_log_error("[%s] cfgfile %s has errors", engine_str, cfgfile);
@@ -1021,22 +972,18 @@ engine_start(const char* cfgfile, int cmdline_verbosity, int daemonize,
         exit(1);
     }
     /* open log */
-    ods_log_init(engine->config->log_filename, engine->config->use_syslog,
-       engine->config->verbosity);
+    ods_log_init("ods-signerd", engine->config->use_syslog,
+        engine->config->log_filename, engine->config->verbosity);
     /* setup */
-    tzset(); /* for portability */
     status = engine_setup(engine);
     if (status != ODS_STATUS_OK) {
         ods_log_error("[%s] setup failed: %s", engine_str,
             ods_status2str(status));
-        engine->need_to_exit = 1;
         if (status != ODS_STATUS_WRITE_PIDFILE_ERR) {
             /* command handler had not yet been started */
             engine->cmdhandler_done = 1;
         }
-    } else {
-        /* setup ok, mark hsm open */
-        close_hsm = 1;
+        goto earlyexit;
     }
 
     /* run */
@@ -1052,25 +999,42 @@ engine_start(const char* cfgfile, int cmdline_verbosity, int daemonize,
         /* start/reload */
         if (engine->need_to_reload) {
             ods_log_info("[%s] signer reloading", engine_str);
+            fifoq_wipe(engine->signq);
             engine->need_to_reload = 0;
         } else {
             ods_log_info("[%s] signer started (version %s), pid %u",
                 engine_str, PACKAGE_VERSION, engine->pid);
+            if (hsm_open2(engine->config->repositories, hsm_check_pin) != HSM_OK) {
+                char* error =  hsm_get_error(NULL);
+                if (error != NULL) {
+                    ods_log_error("[%s] %s", "hsm", error);
+                    free(error);
+                }
+                ods_log_error("[%s] opening hsm failed (for engine recover)", engine_str);
+                break;
+            }
             zl_changed = engine_recover(engine);
+            hsm_close();
         }
         if (zl_changed == ODS_STATUS_OK ||
             zl_changed == ODS_STATUS_UNCHANGED) {
             engine_update_zones(engine, zl_changed);
         }
+        if (hsm_open2(engine->config->repositories, hsm_check_pin) != HSM_OK) {
+            char* error =  hsm_get_error(NULL);
+            if (error != NULL) {
+                ods_log_error("[%s] %s", "hsm", error);
+                free(error);
+            }
+            ods_log_error("[%s] opening hsm failed (for engine run)", engine_str);
+            break;
+        }
         engine_run(engine, single_run);
+        hsm_close();
     }
 
     /* shutdown */
     ods_log_info("[%s] signer shutdown", engine_str);
-    if (close_hsm) {
-        ods_log_verbose("[%s] close hsm", engine_str);
-        hsm_close();
-    }
     engine_stop_cmdhandler(engine);
     engine_stop_xfrhandler(engine);
     engine_stop_dnshandler(engine);
@@ -1087,11 +1051,8 @@ earlyexit:
     tsig_handler_cleanup();
     engine_cleanup(engine);
     engine = NULL;
-    ods_log_close();
-    xmlCleanupParser();
-    xmlCleanupGlobals();
-    xmlCleanupThreads();
-    return;
+
+    return 1;
 }
 
 
@@ -1103,26 +1064,21 @@ void
 engine_cleanup(engine_type* engine)
 {
     size_t i = 0;
-    allocator_type* allocator;
-    cond_basic_type signal_cond;
-    lock_basic_type signal_lock;
+
     if (!engine) {
         return;
     }
-    allocator = engine->allocator;
-    signal_cond = engine->signal_cond;
-    signal_lock = engine->signal_lock;
     if (engine->workers && engine->config) {
         for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
             worker_cleanup(engine->workers[i]);
         }
-        allocator_deallocate(allocator, (void*) engine->workers);
+        free(engine->workers);
     }
     if (engine->drudgers && engine->config) {
        for (i=0; i < (size_t) engine->config->num_signer_threads; i++) {
            worker_cleanup(engine->drudgers[i]);
        }
-        allocator_deallocate(allocator, (void*) engine->drudgers);
+       free(engine->drudgers);
     }
     zonelist_cleanup(engine->zonelist);
     schedule_cleanup(engine->taskq);
@@ -1131,9 +1087,7 @@ engine_cleanup(engine_type* engine)
     dnshandler_cleanup(engine->dnshandler);
     xfrhandler_cleanup(engine->xfrhandler);
     engine_config_cleanup(engine->config);
-    allocator_deallocate(allocator, (void*) engine);
-    lock_basic_destroy(&signal_lock);
-    lock_basic_off(&signal_cond);
-    allocator_cleanup(allocator);
-    return;
+    lock_basic_destroy(&engine->signal_lock);
+    lock_basic_off(&engine->signal_cond);
+    free(engine);
 }
