@@ -35,6 +35,7 @@
 #include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <libxml/parser.h>
 
 
 #define AUTHOR_NAME "Matthijs Mekking"
@@ -80,6 +81,28 @@ version(FILE* out)
     exit(0);
 }
 
+static void
+program_setup(int cmdline_verbosity)
+{
+    ods_log_init("ods-signerd", 0, NULL, cmdline_verbosity);
+    ods_log_verbose("[engine] starting signer");
+
+    /* initialize */
+    xmlInitGlobals();
+    xmlInitParser();
+    xmlInitThreads();
+
+    tzset(); /* for portability */
+}
+
+static void
+program_teardown()
+{
+    xmlCleanupParser();
+    xmlCleanupGlobals();
+    xmlCleanupThreads();
+    ods_log_close();
+}
 
 /**
  * Main. start engine and run it.
@@ -88,12 +111,13 @@ version(FILE* out)
 int
 main(int argc, char* argv[])
 {
-    int c;
+    int c, returncode;
     int options_index = 0;
     int info = 0;
     int single_run = 0;
     int daemonize = 1;
     int cmdline_verbosity = 0;
+    char *time_arg = NULL;
     const char* cfgfile = ODS_SE_CFGFILE;
     static struct option long_options[] = {
         {"single-run", no_argument, 0, '1'},
@@ -103,6 +127,7 @@ main(int argc, char* argv[])
         {"info", no_argument, 0, 'i'},
         {"verbose", no_argument, 0, 'v'},
         {"version", no_argument, 0, 'V'},
+        {"set-time", required_argument, 0, 256},
         { 0, 0, 0, 0}
     };
 
@@ -133,6 +158,9 @@ main(int argc, char* argv[])
                 version(stdout);
                 exit(0);
                 break;
+            case 256:
+                time_arg = optarg;
+                break;
             default:
                 usage(stderr);
                 exit(2);
@@ -146,20 +174,20 @@ main(int argc, char* argv[])
         exit(2);
     }
 
-#ifdef ENFORCER_TIMESHIFT
-    if (getenv("ENFORCER_TIMESHIFT")) {
-        fprintf(stdout, "WARNING: timeshift %s detected, running once only\n",
-            getenv("ENFORCER_TIMESHIFT"));
-        single_run = 1;
-    } else {
-        fprintf(stdout, "DEBUG: timeshift mode enabled, but not set.\n");
+    if (time_arg) {
+        if(set_time_now_str(time_arg)) {
+            fprintf(stderr, "Error: Failed to interpret start time argument.  Daemon not started.\n");
+            return 1;
+        }
     }
-#endif /* ENFORCER_TIMESHIFT */
 
     /* main stuff */
     fprintf(stdout, "OpenDNSSEC signer engine version %s\n", PACKAGE_VERSION);
-    engine_start(cfgfile, cmdline_verbosity, daemonize, info, single_run);
 
-    /* done */
-    return 0;
+    program_setup(cmdline_verbosity);
+    returncode = engine_start(cfgfile, cmdline_verbosity, daemonize,
+        info, single_run);
+    program_teardown();
+
+    return returncode;
 }
