@@ -32,8 +32,8 @@
 #include "config.h"
 #include "scheduler/schedule.h"
 #include "scheduler/task.h"
-#include "shared/duration.h"
-#include "shared/log.h"
+#include "duration.h"
+#include "log.h"
 
 #include <ldns/ldns.h>
 
@@ -45,27 +45,22 @@ static const char* schedule_str = "scheduler";
  *
  */
 schedule_type*
-schedule_create(allocator_type* allocator)
+schedule_create()
 {
     schedule_type* schedule;
-    if (!allocator) {
-        return NULL;
-    }
-    schedule = (schedule_type*) allocator_alloc(allocator,
-        sizeof(schedule_type));
+    CHECKALLOC(schedule = (schedule_type*) malloc(sizeof(schedule_type)));
     if (!schedule) {
         ods_log_error("[%s] unable to create schedule: allocator_alloc() "
             "failed", schedule_str);
         return NULL;
     }
-    schedule->allocator = allocator;
     schedule->loading = 0;
     schedule->flushcount = 0;
     schedule->tasks = ldns_rbtree_create(task_compare);
     if (!schedule->tasks) {
         ods_log_error("[%s] unable to create schedule: ldns_rbtree_create() "
             "failed", schedule_str);
-        allocator_deallocate(allocator, (void*) schedule);
+        free(schedule);
         return NULL;
     }
     lock_basic_init(&schedule->schedule_lock);
@@ -97,7 +92,6 @@ schedule_flush(schedule_type* schedule, task_id override)
         }
         node = ldns_rbtree_next(node);
     }
-    return;
 }
 
 
@@ -215,28 +209,6 @@ unschedule_task(schedule_type* schedule, task_type* task)
 }
 
 
-/**
- * Reschedule task.
- *
- */
-ods_status
-reschedule_task(schedule_type* schedule, task_type* task, task_id what,
-    time_t when)
-{
-    task_type* del_task = NULL;
-    if (!task || !schedule || !schedule->tasks) {
-        return ODS_STATUS_ASSERT_ERR;
-    }
-    del_task = unschedule_task(schedule, task);
-    if (!del_task) {
-        del_task = task;
-    }
-    del_task->what = what;
-    del_task->when = when;
-    return schedule_task(schedule, del_task, 1);
-}
-
-
 /**
  * Get the first scheduled task.
  *
@@ -305,30 +277,6 @@ schedule_pop_task(schedule_type* schedule)
 }
 
 
-/**
- * Print schedule.
- *
- */
-void
-schedule_print(FILE* out, schedule_type* schedule)
-{
-    ldns_rbnode_t* node = LDNS_RBTREE_NULL;
-    task_type* task = NULL;
-
-    if (!out || !schedule || !schedule->tasks) {
-        return;
-    }
-    node = ldns_rbtree_first(schedule->tasks);
-    while (node && node != LDNS_RBTREE_NULL) {
-        task = (task_type*) node->data;
-        task_print(out, task);
-        node = ldns_rbtree_next(node);
-    }
-    fprintf(out, "\n");
-    return;
-}
-
-
 /**
  * Internal task cleanup function.
  *
@@ -345,7 +293,6 @@ task_delfunc(ldns_rbnode_t* elem)
         task_cleanup(task);
         free((void*)elem);
     }
-    return;
 }
 
 
@@ -356,9 +303,6 @@ task_delfunc(ldns_rbnode_t* elem)
 void
 schedule_cleanup(schedule_type* schedule)
 {
-    allocator_type* allocator;
-    lock_basic_type schedule_lock;
-
     if (!schedule) {
         return;
     }
@@ -368,9 +312,6 @@ schedule_cleanup(schedule_type* schedule)
         ldns_rbtree_free(schedule->tasks);
         schedule->tasks = NULL;
     }
-    allocator = schedule->allocator;
-    schedule_lock = schedule->schedule_lock;
-    allocator_deallocate(allocator, (void*) schedule);
-    lock_basic_destroy(&schedule_lock);
-    return;
+    lock_basic_destroy(&schedule->schedule_lock);
+    free(schedule);
 }
