@@ -30,17 +30,16 @@
  */
 
 #include "config.h"
-#include "shared/allocator.h"
-#include "shared/file.h"
-#include "shared/log.h"
-#include "shared/util.h"
+#include "status.h"
+#include "file.h"
+#include "log.h"
+#include "util.h"
 #include "signer/backup.h"
 #include "signer/namedb.h"
 #include "signer/zone.h"
 
 const char* db_str = "namedb";
 
-
 /**
  * Convert a domain to a tree node.
  *
@@ -98,7 +97,6 @@ namedb_init_denials(namedb_type* db)
     if (db) {
         db->denials = ldns_rbtree_create(domain_compare);
     }
-    return;
 }
 
 
@@ -112,7 +110,6 @@ namedb_init_domains(namedb_type* db)
     if (db) {
         db->domains = ldns_rbtree_create(domain_compare);
     }
-    return;
 }
 
 
@@ -128,8 +125,7 @@ namedb_create(void* zone)
 
     ods_log_assert(z);
     ods_log_assert(z->name);
-    ods_log_assert(z->allocator);
-    db = (namedb_type*) allocator_alloc(z->allocator, sizeof(namedb_type));
+    CHECKALLOC(db = (namedb_type*) malloc(sizeof(namedb_type)));
     if (!db) {
         ods_log_error("[%s] unable to create namedb for zone %s: "
             "allocator_alloc() failed", db_str, z->name);
@@ -237,7 +233,7 @@ namedb_update_serial(namedb_type* db, const char* zone_name, const char* format,
         soa = (uint32_t) time_datestamp(0, "%Y%m%d", NULL) * 100;
         if (!util_serial_gt(soa, prev)) {
             if (!db->have_serial) {
-                ods_log_warning("[%s] zone %s unable to use datecounter as "
+                ods_log_info("[%s] zone %s unable to use datecounter as "
                     "serial: %u does not increase %u. Serial set to %u", db_str,
                     zone_name, soa, prev, (prev+1));
             }
@@ -500,7 +496,6 @@ namedb_add_nsec_trigger(namedb_type* db, domain_type* domain)
     denial->domain = (void*) domain;
     domain->denial = (void*) denial;
     domain->is_new = 0;
-    return;
 }
 
 
@@ -537,7 +532,6 @@ namedb_add_nsec3_trigger(namedb_type* db, domain_type* domain,
     denial->domain = (void*) domain;
     domain->denial = (void*) denial;
     domain->is_new = 0;
-    return;
 }
 
 
@@ -552,17 +546,18 @@ namedb_add_denial_trigger(namedb_type* db, domain_type* domain)
     ods_log_assert(db);
     ods_log_assert(domain);
     if (!domain->denial) {
-        zone = (void*) domain->zone;
+        zone = domain->zone;
         ods_log_assert(zone);
         ods_log_assert(zone->signconf);
-        if (zone->signconf->nsec_type == LDNS_RR_TYPE_NSEC) {
-            namedb_add_nsec_trigger(db, domain);
-        } else {
-            ods_log_assert(zone->signconf->nsec_type == LDNS_RR_TYPE_NSEC3);
-            namedb_add_nsec3_trigger(db, domain, zone->signconf->nsec3params);
+        if (!zone->signconf->passthrough) {
+            if (zone->signconf->nsec_type == LDNS_RR_TYPE_NSEC) {
+                namedb_add_nsec_trigger(db, domain);
+            } else {
+                ods_log_assert(zone->signconf->nsec_type == LDNS_RR_TYPE_NSEC3);
+                namedb_add_nsec3_trigger(db, domain, zone->signconf->nsec3params);
+            }
         }
     }
-    return;
 }
 
 
@@ -587,7 +582,6 @@ namedb_del_nsec_trigger(namedb_type* db, domain_type* domain)
        denial_cleanup(denial);
        domain->denial = NULL;
     }
-    return;
 }
 
 
@@ -624,7 +618,6 @@ namedb_del_nsec3_trigger(namedb_type* db, domain_type* domain,
             domain->denial = NULL;
         }
     }
-    return;
 }
 
 
@@ -641,7 +634,7 @@ namedb_del_denial_trigger(namedb_type* db, domain_type* domain, int rollback)
     ods_log_assert(db);
     ods_log_assert(domain);
     ods_log_assert(domain->dname);
-    zone = (void*) domain->zone;
+    zone = domain->zone;
     ods_log_assert(zone);
     ods_log_assert(zone->signconf);
     while(domain) {
@@ -783,8 +776,8 @@ namedb_del_denial(namedb_type* db, denial_type* denial)
         return NULL;
     }
     if (denial->rrset && denial->rrset->rr_count) {
-        ods_log_error("[%s] unable to delete denial: denial in use [#%u]",
-            db_str, denial->rrset->rr_count);
+        ods_log_error("[%s] unable to delete denial: denial in use [#%lu]",
+            db_str, (unsigned long)denial->rrset->rr_count);
         log_dname(denial->dname, "ERR -DENIAL", LOG_ERR);
         return NULL;
     }
@@ -844,7 +837,6 @@ namedb_diff(namedb_type* db, unsigned is_ixfr, unsigned more_coming)
             namedb_add_denial_trigger(db, domain);
         }
     }
-    return;
 }
 
 
@@ -870,7 +862,6 @@ namedb_rollback(namedb_type* db, unsigned keepsc)
         domain_rollback(domain, keepsc);
         (void) namedb_del_denial_trigger(db, domain, 1);
     }
-    return;
 }
 
 
@@ -901,7 +892,6 @@ namedb_nsecify(namedb_type* db, uint32_t* num_added)
     if (num_added) {
         *num_added = nsec_added;
     }
-    return;
 }
 
 
@@ -1018,23 +1008,14 @@ namedb_wipe_denial(namedb_type* db)
                 rrset_del_rr(denial->rrset, i);
                 i--;
             }
-            for (i=0; i < denial->rrset->rrsig_count; i++) {
-                /* ixfr -RRSIG */
-                lock_basic_lock(&zone->ixfr->ixfr_lock);
-                ixfr_del_rr(zone->ixfr, denial->rrset->rrsigs[i].rr);
-                lock_basic_unlock(&zone->ixfr->ixfr_lock);
-                rrset_del_rrsig(denial->rrset, i);
-                i--;
-            }
+            rrset_drop_rrsigs(zone, denial->rrset);
             rrset_cleanup(denial->rrset);
             denial->rrset = NULL;
             node = ldns_rbtree_next(node);
         }
     }
-    return;
 }
 
-
 /**
  * Export db to file.
  *
@@ -1067,7 +1048,6 @@ namedb_export(FILE* fd, namedb_type* db, ods_status* status)
         }
         node = ldns_rbtree_next(node);
     }
-    return;
 }
 
 
@@ -1086,7 +1066,6 @@ domain_delfunc(ldns_rbnode_t* elem)
         domain_cleanup(domain);
         free((void*)elem);
     }
-    return;
 }
 
 
@@ -1110,7 +1089,6 @@ denial_delfunc(ldns_rbnode_t* elem)
         denial_cleanup(denial);
         free((void*)elem);
     }
-    return;
 }
 
 
@@ -1126,7 +1104,6 @@ namedb_cleanup_domains(namedb_type* db)
         ldns_rbtree_free(db->domains);
         db->domains = NULL;
     }
-    return;
 }
 
 
@@ -1142,7 +1119,6 @@ namedb_cleanup_denials(namedb_type* db)
         ldns_rbtree_free(db->denials);
         db->denials = NULL;
     }
-    return;
 }
 
 
@@ -1158,13 +1134,12 @@ namedb_cleanup(namedb_type* db)
         return;
     }
     z = (zone_type*) db->zone;
-    if (!z || !z->allocator) {
+    if (!z) {
         return;
     }
     namedb_cleanup_denials(db);
     namedb_cleanup_domains(db);
-    allocator_deallocate(z->allocator, (void*) db);
-    return;
+    free(db);
 }
 
 
@@ -1213,5 +1188,4 @@ namedb_backup2(FILE* fd, namedb_type* db)
         node = ldns_rbtree_next(node);
     }
     fprintf(fd, ";\n");
-    return;
 }
