@@ -30,16 +30,16 @@
  */
 
 #include "config.h"
-#include "shared/file.h"
-#include "shared/hsm.h"
-#include "shared/log.h"
-#include "shared/util.h"
+#include "file.h"
+#include "hsm.h"
+#include "log.h"
+#include "util.h"
+#include "compat.h"
 #include "signer/rrset.h"
 #include "signer/zone.h"
 
 static const char* rrset_str = "rrset";
 
-
 /**
  * Log RR.
  *
@@ -88,7 +88,6 @@ log_rr(ldns_rr* rr, const char* pre, int level)
         ods_log_deeebug("[%s] %s: %s", rrset_str, pre?pre:"", str);
     }
     free((void*)str);
-    return;
 }
 
 
@@ -148,7 +147,6 @@ log_rrset(ldns_rdf* dname, ldns_rr_type type, const char* pre, int level)
             rrset_type2str(type));
     }
     free((void*)str);
-    return;
 }
 
 
@@ -178,21 +176,32 @@ rrset_type2str(ldns_rr_type type)
     return "TYPE???";
 }
 
+static int
+memberdestroy(void* dummy, void* member)
+{
+    rrsig_type* sig = (rrsig_type*) member;
+    (void)dummy;
+    free((void*) sig->key_locator);
+    sig->key_locator = NULL;
+    /* The rrs may still be in use by IXFRs so cannot do ldns_rr_free(sig->rr); */
+    sig->owner = NULL;
+    sig->rr = NULL;
+    return 0;
+}
+
 
 /**
  * Create RRset.
  *
  */
 rrset_type*
-rrset_create(void* zoneptr, ldns_rr_type type)
+rrset_create(zone_type* zone, ldns_rr_type type)
 {
-    zone_type* zone = (zone_type*) zoneptr;
     rrset_type* rrset = NULL;
-    if (!type || !zoneptr) {
+    if (!type || !zone) {
         return NULL;
     }
-    rrset = (rrset_type*) allocator_alloc(
-        zone->allocator, sizeof(rrset_type));
+    CHECKALLOC(rrset = (rrset_type*) malloc(sizeof(rrset_type)));
     if (!rrset) {
         ods_log_error("[%s] unable to create RRset %u: allocator_alloc() "
             "failed", rrset_str, (unsigned) type);
@@ -200,16 +209,23 @@ rrset_create(void* zoneptr, ldns_rr_type type)
     }
     rrset->next = NULL;
     rrset->rrs = NULL;
-    rrset->rrsigs = NULL;
     rrset->domain = NULL;
-    rrset->zone = zoneptr;
+    rrset->zone = zone;
     rrset->rrtype = type;
     rrset->rr_count = 0;
-    rrset->rrsig_count = 0;
+    collection_create_array(&rrset->rrsigs, sizeof(rrsig_type), rrset->zone->rrstore);
     rrset->needs_signing = 0;
     return rrset;
 }
 
+collection_class
+rrset_store_initialize()
+{
+    collection_class klass;
+    collection_class_allocated(&klass, NULL, memberdestroy);
+    return klass;
+}
+
 
 /**
  * Lookup RR in RRset.
@@ -269,16 +285,13 @@ rr_type*
 rrset_add_rr(rrset_type* rrset, ldns_rr* rr)
 {
     rr_type* rrs_old = NULL;
-    zone_type* zone = NULL;
 
     ods_log_assert(rrset);
     ods_log_assert(rr);
     ods_log_assert(rrset->rrtype == ldns_rr_get_type(rr));
 
-    zone = (zone_type*) rrset->zone;
     rrs_old = rrset->rrs;
-    rrset->rrs = (rr_type*) allocator_alloc(zone->allocator,
-        (rrset->rr_count + 1) * sizeof(rr_type));
+    CHECKALLOC(rrset->rrs = (rr_type*) malloc((rrset->rr_count + 1) * sizeof(rr_type)));
     if (!rrset->rrs) {
         ods_fatal_exit("[%s] fatal unable to add RR: allocator_alloc() failed",
             rrset_str);
@@ -286,7 +299,7 @@ rrset_add_rr(rrset_type* rrset, ldns_rr* rr)
     if (rrs_old) {
         memcpy(rrset->rrs, rrs_old, (rrset->rr_count) * sizeof(rr_type));
     }
-    allocator_deallocate(zone->allocator, (void*) rrs_old);
+    free(rrs_old);
     rrset->rr_count++;
     rrset->rrs[rrset->rr_count - 1].owner = rrset->domain;
     rrset->rrs[rrset->rr_count - 1].rr = rr;
@@ -307,12 +320,10 @@ void
 rrset_del_rr(rrset_type* rrset, uint16_t rrnum)
 {
     rr_type* rrs_orig = NULL;
-    zone_type* zone = NULL;
 
     ods_log_assert(rrset);
     ods_log_assert(rrnum < rrset->rr_count);
 
-    zone = (zone_type*) rrset->zone;
     log_rr(rrset->rrs[rrnum].rr, "-RR", LOG_DEEEBUG);
     rrset->rrs[rrnum].owner = NULL;
     rrset->rrs[rrnum].rr = NULL;
@@ -322,20 +333,17 @@ rrset_del_rr(rrset_type* rrset, uint16_t rrnum)
     }
     memset(&rrset->rrs[rrset->rr_count-1], 0, sizeof(rr_type));
     rrs_orig = rrset->rrs;
-    rrset->rrs = (rr_type*) allocator_alloc(zone->allocator,
-        (rrset->rr_count - 1) * sizeof(rr_type));
+    CHECKALLOC(rrset->rrs = (rr_type*) malloc((rrset->rr_count - 1) * sizeof(rr_type)));
     if(!rrset->rrs) {
         ods_fatal_exit("[%s] fatal unable to delete RR: allocator_alloc() failed",
             rrset_str);
     }
     memcpy(rrset->rrs, rrs_orig, (rrset->rr_count -1) * sizeof(rr_type));
-    allocator_deallocate(zone->allocator, (void*) rrs_orig);
+    free(rrs_orig);
     rrset->rr_count--;
     rrset->needs_signing = 1;
-    return;
 }
 
-
 /**
  * Apply differences at RRset.
  *
@@ -379,93 +387,46 @@ rrset_diff(rrset_type* rrset, unsigned is_ixfr, unsigned more_coming)
         }
     }
     if (del_sigs) {
-       for (i=0; i < rrset->rrsig_count; i++) {
-            /* ixfr -RRSIG */
-            lock_basic_lock(&zone->ixfr->ixfr_lock);
-            ixfr_del_rr(zone->ixfr, rrset->rrsigs[i].rr);
-            lock_basic_unlock(&zone->ixfr->ixfr_lock);
-            rrset_del_rrsig(rrset, i);
-            i--;
-        }
+        rrset_drop_rrsigs(zone, rrset);
     }
-    return;
 }
 
+/**
+ * Remove signatures, deallocate storage and add then to the outgoing IFXR for that zone.
+ *
+ */
+void
+rrset_drop_rrsigs(zone_type* zone, rrset_type* rrset)
+{
+    rrsig_type* rrsig;
+    while((rrsig = collection_iterator(rrset->rrsigs))) {
+        /* ixfr -RRSIG */
+        lock_basic_lock(&zone->ixfr->ixfr_lock);
+        ixfr_del_rr(zone->ixfr, rrsig->rr);
+        lock_basic_unlock(&zone->ixfr->ixfr_lock);
+        collection_del_cursor(rrset->rrsigs);
+    }
+}
 
 /**
  * Add RRSIG to RRset.
  *
  */
-rrsig_type*
+void
 rrset_add_rrsig(rrset_type* rrset, ldns_rr* rr,
     const char* locator, uint32_t flags)
 {
-    rrsig_type* rrsigs_old = NULL;
-    zone_type* zone = NULL;
+    rrsig_type rrsig;
     ods_log_assert(rrset);
     ods_log_assert(rr);
     ods_log_assert(ldns_rr_get_type(rr) == LDNS_RR_TYPE_RRSIG);
-    zone = (zone_type*) rrset->zone;
-    rrsigs_old = rrset->rrsigs;
-    rrset->rrsigs = (rrsig_type*) allocator_alloc(zone->allocator,
-        (rrset->rrsig_count + 1) * sizeof(rrsig_type));
-    if (!rrset->rrsigs) {
-        ods_fatal_exit("[%s] fatal unable to add RRSIG: allocator_alloc() failed",
-            rrset_str);
-    }
-    if (rrsigs_old) {
-        memcpy(rrset->rrsigs, rrsigs_old,
-            (rrset->rrsig_count) * sizeof(rrsig_type));
-    }
-    allocator_deallocate(zone->allocator, (void*) rrsigs_old);
-    rrset->rrsig_count++;
-    rrset->rrsigs[rrset->rrsig_count - 1].owner = rrset->domain;
-    rrset->rrsigs[rrset->rrsig_count - 1].rr = rr;
-    rrset->rrsigs[rrset->rrsig_count - 1].key_locator = locator;
-    rrset->rrsigs[rrset->rrsig_count - 1].key_flags = flags;
-    log_rr(rr, "+RRSIG", LOG_DEEEBUG);
-    return &rrset->rrsigs[rrset->rrsig_count -1];
+    rrsig.owner = rrset->domain;
+    rrsig.rr = rr;
+    rrsig.key_locator = locator;
+    rrsig.key_flags = flags;
+    collection_add(rrset->rrsigs, &rrsig);
 }
 
-
-/**
- * Delete RRSIG from RRset.
- *
- */
-void
-rrset_del_rrsig(rrset_type* rrset, uint16_t rrnum)
-{
-    rrsig_type* rrsigs_orig = NULL;
-    zone_type* zone = NULL;
-    ods_log_assert(rrset);
-    ods_log_assert(rrnum < rrset->rrsig_count);
-    zone = (zone_type*) rrset->zone;
-    log_rr(rrset->rrsigs[rrnum].rr, "-RRSIG", LOG_DEEEBUG);
-    rrset->rrsigs[rrnum].owner = NULL;
-    rrset->rrsigs[rrnum].rr = NULL;
-    allocator_deallocate(zone->allocator,
-        (void*)rrset->rrsigs[rrnum].key_locator);
-    rrset->rrsigs[rrnum].key_locator = NULL;
-    while (rrnum < rrset->rrsig_count-1) {
-        rrset->rrsigs[rrnum] = rrset->rrsigs[rrnum+1];
-        rrnum++;
-    }
-    memset(&rrset->rrsigs[rrset->rrsig_count-1], 0, sizeof(rrsig_type));
-    rrsigs_orig = rrset->rrsigs;
-    rrset->rrsigs = (rrsig_type*) allocator_alloc(zone->allocator,
-        (rrset->rrsig_count - 1) * sizeof(rrsig_type));
-    if(!rrset->rrsigs) {
-        ods_fatal_exit("[%s] fatal unable to delete RRSIG: allocator_alloc() failed",
-            rrset_str);
-    }
-    memcpy(rrset->rrsigs, rrsigs_orig,
-        (rrset->rrsig_count -1) * sizeof(rrsig_type));
-    allocator_deallocate(zone->allocator, (void*) rrsigs_orig);
-    rrset->rrsig_count--;
-    return;
-}
-
-
 /**
  * Recycle signatures from RRset and drop unreusable signatures.
  *
@@ -479,9 +440,9 @@ rrset_recycle(rrset_type* rrset, time_t signtime, ldns_rr_type dstatus,
     uint32_t inception = 0;
     uint32_t reusedsigs = 0;
     unsigned drop_sig = 0;
-    size_t i = 0;
     key_type* key = NULL;
     zone_type* zone = NULL;
+    rrsig_type* rrsig;
 
     if (!rrset) {
         return 0;
@@ -493,7 +454,7 @@ rrset_recycle(rrset_type* rrset, time_t signtime, ldns_rr_type dstatus,
             duration2time(zone->signconf->sig_refresh_interval));
     }
     /* Check every signature if it matches the recycling logic. */
-    for (i=0; i < rrset->rrsig_count; i++) {
+    while((rrsig = collection_iterator(rrset->rrsigs))) {
         drop_sig = 0;
         /* 0. Skip delegation, glue and occluded RRsets */
         if (dstatus != LDNS_RR_TYPE_SOA || (delegpt != LDNS_RR_TYPE_SOA &&
@@ -511,22 +472,22 @@ rrset_recycle(rrset_type* rrset, time_t signtime, ldns_rr_type dstatus,
         }
         /* 3. Expiration - Refresh has passed */
         expiration = ldns_rdf2native_int32(
-            ldns_rr_rrsig_expiration(rrset->rrsigs[i].rr));
+            ldns_rr_rrsig_expiration(rrsig->rr));
         if (expiration < refresh) {
             drop_sig = 1;
             goto recycle_drop_sig;
         }
         /* 4. Inception has not yet passed */
         inception = ldns_rdf2native_int32(
-            ldns_rr_rrsig_inception(rrset->rrsigs[i].rr));
+            ldns_rr_rrsig_inception(rrsig->rr));
         if (inception > (uint32_t) signtime) {
             drop_sig = 1;
             goto recycle_drop_sig;
         }
         /* 5. Corresponding key is dead (key is locator+flags) */
         key = keylist_lookup_by_locator(zone->signconf->keys,
-            rrset->rrsigs[i].key_locator);
-        if (!key || key->flags != rrset->rrsigs[i].key_flags) {
+            rrsig->key_locator);
+        if (!key || key->flags != rrsig->key_flags) {
             drop_sig = 1;
         }
 
@@ -535,10 +496,9 @@ recycle_drop_sig:
             /* A rule mismatched, refresh signature */
             /* ixfr -RRSIG */
             lock_basic_lock(&zone->ixfr->ixfr_lock);
-            ixfr_del_rr(zone->ixfr, rrset->rrsigs[i].rr);
+            ixfr_del_rr(zone->ixfr, rrsig->rr);
             lock_basic_unlock(&zone->ixfr->ixfr_lock);
-            rrset_del_rrsig(rrset, i);
-            i--;
+            collection_del_cursor(rrset->rrsigs);
         } else {
             /* All rules ok, recycle signature */
             reusedsigs += 1;
@@ -555,20 +515,18 @@ recycle_drop_sig:
 static int
 rrset_sigalgo(rrset_type* rrset, uint8_t algorithm)
 {
-    size_t i = 0;
-    if (!rrset) {
-        return 0;
-    }
-    for (i=0; i < rrset->rrsig_count; i++) {
-        if (algorithm == ldns_rdf2native_int8(
-                ldns_rr_rrsig_algorithm(rrset->rrsigs[i].rr))) {
-            return 1;
+    rrsig_type* rrsig;
+    int match = 0;
+    if (rrset) {
+        while((rrsig = collection_iterator(rrset->rrsigs))) {
+            if (algorithm == ldns_rdf2native_int8(ldns_rr_rrsig_algorithm(rrsig->rr))) {
+                match = 1;
+            }
         }
     }
-    return 0;
+    return match;
 }
 
-
 /**
  * Is the RRset signed with this locator?
  *
@@ -576,16 +534,16 @@ rrset_sigalgo(rrset_type* rrset, uint8_t algorithm)
 static int
 rrset_siglocator(rrset_type* rrset, const char* locator)
 {
-    size_t i = 0;
-    if (!rrset) {
-        return 0;
-    }
-    for (i=0; i < rrset->rrsig_count; i++) {
-        if (!ods_strcmp(locator, rrset->rrsigs[i].key_locator)) {
-            return 1;
+    rrsig_type* rrsig;
+    int match = 0;
+    if (rrset) {
+        while ((rrsig = collection_iterator(rrset->rrsigs))) {
+            if (!ods_strcmp(locator, rrsig->key_locator)) {
+                match = 1;
+            }
         }
     }
-    return 0;
+    return match;
 }
 
 
@@ -643,14 +601,23 @@ rrset_sigvalid_period(signconf_type* sc, ldns_rr_type rrtype, time_t signtime,
         random_jitter = ods_rand(jitter*2);
     }
     offset = duration2time(sc->sig_inception_offset);
-    if (rrtype == LDNS_RR_TYPE_NSEC || rrtype == LDNS_RR_TYPE_NSEC3) {
-        validity = duration2time(sc->sig_validity_denial);
-    } else {
-        validity = duration2time(sc->sig_validity_default);
+    switch (rrtype) {
+        case LDNS_RR_TYPE_NSEC:
+        case LDNS_RR_TYPE_NSEC3:
+            validity = duration2time(sc->sig_validity_denial);
+            break;
+        case LDNS_RR_TYPE_DNSKEY:
+            if (sc->sig_validity_keyset != NULL && duration2time(sc->sig_validity_keyset) > 0) {
+                validity = duration2time(sc->sig_validity_keyset);
+            } else {
+                validity = duration2time(sc->sig_validity_default);
+            }
+            break;
+        default:
+            validity = duration2time(sc->sig_validity_default);
     }
     *inception = signtime - offset;
     *expiration = (signtime + validity + random_jitter) - jitter;
-    return;
 }
 
 
@@ -661,12 +628,12 @@ rrset_sigvalid_period(signconf_type* sc, ldns_rr_type rrtype, time_t signtime,
 ods_status
 rrset_sign(hsm_ctx_t* ctx, rrset_type* rrset, time_t signtime)
 {
+    ods_status status;
     zone_type* zone = NULL;
     uint32_t newsigs = 0;
     uint32_t reusedsigs = 0;
     ldns_rr* rrsig = NULL;
     ldns_rr_list* rr_list = NULL;
-    rrsig_type* signature = NULL;
     const char* locator = NULL;
     time_t inception = 0;
     time_t expiration = 0;
@@ -714,11 +681,6 @@ rrset_sign(hsm_ctx_t* ctx, rrset_type* rrset, time_t signtime)
         (delegpt == LDNS_RR_TYPE_SOA || rrset->rrtype == LDNS_RR_TYPE_DS));
     /* Transmogrify rrset */
     rr_list = rrset2rrlist(rrset);
-    if (!rr_list) {
-        ods_log_error("[%s] unable to sign RRset[%i]: rrset2rrlist() failed",
-            rrset_str, rrset->rrtype);
-        return ODS_STATUS_MALLOC_ERR;
-    }
     if (ldns_rr_list_rr_count(rr_list) <= 0) {
         /* Empty RRset, no signatures needed */
         ldns_rr_list_free(rr_list);
@@ -747,11 +709,10 @@ rrset_sign(hsm_ctx_t* ctx, rrset_type* rrset, time_t signtime)
 	    rrset_sigalgo(rrset, zone->signconf->keys->keys[i].algorithm)) {
             continue;
         }
-
-        /**
-         * currently, there is no rule that the number of signatures
-         * over this RRset equals the number of active keys.
-         */
+        /* If key has no locator, and should be pre-signed dnskey RR, skip */
+        if (zone->signconf->keys->keys[i].ksk && zone->signconf->keys->keys[i].locator == NULL) {
+            continue;
+        }
 
         /* Sign the RRset with this key */
         ods_log_deeebug("[%s] signing RRset[%i] with key %s", rrset_str,
@@ -765,17 +726,32 @@ rrset_sign(hsm_ctx_t* ctx, rrset_type* rrset, time_t signtime)
             return ODS_STATUS_HSM_ERR;
         }
         /* Add signature */
-        locator = allocator_strdup(zone->allocator,
-            zone->signconf->keys->keys[i].locator);
-        signature = rrset_add_rrsig(rrset, rrsig, locator,
+        locator = strdup(zone->signconf->keys->keys[i].locator);
+        rrset_add_rrsig(rrset, rrsig, locator,
             zone->signconf->keys->keys[i].flags);
         newsigs++;
         /* ixfr +RRSIG */
-        ods_log_assert(signature->rr);
         lock_basic_lock(&zone->ixfr->ixfr_lock);
-        ixfr_add_rr(zone->ixfr, signature->rr);
+        ixfr_add_rr(zone->ixfr, rrsig);
         lock_basic_unlock(&zone->ixfr->ixfr_lock);
     }
+    if(rrset->rrtype == LDNS_RR_TYPE_DNSKEY && zone->signconf->dnskey_signature) {
+        for(i=0; zone->signconf->dnskey_signature[i]; i++) {
+            rrsig = NULL;
+            if ((status = rrset_getliteralrr(&rrsig, zone->signconf->dnskey_signature[i], duration2time(zone->signconf->dnskey_ttl), zone->apex)) != ODS_STATUS_OK) {
+                    ods_log_error("[%s] unable to publish dnskeys for zone %s: "
+                            "error decoding literal dnskey", rrset_str, zone->name);
+                    return status;
+            }
+            /* Add signature */
+            rrset_add_rrsig(rrset, rrsig, NULL, 0);
+            newsigs++;
+            /* ixfr +RRSIG */
+            lock_basic_lock(&zone->ixfr->ixfr_lock);
+            ixfr_add_rr(zone->ixfr, rrsig);
+            lock_basic_unlock(&zone->ixfr->ixfr_lock);            
+        }
+    }
     /* RRset signing completed */
     ldns_rr_list_free(rr_list);
     lock_basic_lock(&zone->stats->stats_lock);
@@ -788,6 +764,21 @@ rrset_sign(hsm_ctx_t* ctx, rrset_type* rrset, time_t signtime)
     return ODS_STATUS_OK;
 }
 
+ods_status
+rrset_getliteralrr(ldns_rr** dnskey, const char *resourcerecord, uint32_t ttl, ldns_rdf* apex)
+{
+    uint8_t dnskeystring[4096];
+    ldns_status ldnsstatus;
+    int len;
+    if ((len = b64_pton(resourcerecord, dnskeystring, sizeof (dnskeystring) - 2)) < 0) {
+        return ODS_STATUS_PARSE_ERR;
+    }
+    dnskeystring[len] = '\0';
+    if ((ldnsstatus = ldns_rr_new_frm_str(dnskey, (const char*) dnskeystring, ttl, apex, NULL)) != LDNS_STATUS_OK) {
+        return ODS_STATUS_PARSE_ERR;
+    }
+    return ODS_STATUS_OK;
+}
 
 /**
  * Print RRset.
@@ -797,6 +788,7 @@ void
 rrset_print(FILE* fd, rrset_type* rrset, int skip_rrsigs,
     ods_status* status)
 {
+    rrsig_type* rrsig;
     uint16_t i = 0;
     ods_status result = ODS_STATUS_OK;
 
@@ -806,41 +798,42 @@ rrset_print(FILE* fd, rrset_type* rrset, int skip_rrsigs,
         if (status) {
             *status = ODS_STATUS_ASSERT_ERR;
         }
-        return;
-    }
-    for (i=0; i < rrset->rr_count; i++) {
-        if (rrset->rrs[i].exists) {
-            result = util_rr_print(fd, rrset->rrs[i].rr);
-            if (rrset->rrtype == LDNS_RR_TYPE_CNAME ||
-                rrset->rrtype == LDNS_RR_TYPE_DNAME) {
-                /* singleton types */
-                break;
-            }
-            if (result != ODS_STATUS_OK) {
-                zone_type* zone = (zone_type*) rrset->zone;
-                log_rrset(ldns_rr_owner(rrset->rrs[i].rr), rrset->rrtype,
-                    "error printing RRset", LOG_CRIT);
-                zone->adoutbound->error = 1;
-                break;
+    } else {
+        for (i=0; i < rrset->rr_count; i++) {
+            if (rrset->rrs[i].exists) {
+                result = util_rr_print(fd, rrset->rrs[i].rr);
+                if (rrset->rrtype == LDNS_RR_TYPE_CNAME ||
+                    rrset->rrtype == LDNS_RR_TYPE_DNAME) {
+                    /* singleton types */
+                    break;
+                }
+                if (result != ODS_STATUS_OK) {
+                    zone_type* zone = (zone_type*) rrset->zone;
+                    log_rrset(ldns_rr_owner(rrset->rrs[i].rr), rrset->rrtype,
+                        "error printing RRset", LOG_CRIT);
+                    zone->adoutbound->error = 1;
+                    break;
+                }
             }
         }
-    }
-    if (! (skip_rrsigs || !rrset->rrsig_count)) {
-        for (i=0; i < rrset->rrsig_count; i++) {
-            result = util_rr_print(fd, rrset->rrsigs[i].rr);
-            if (result != ODS_STATUS_OK) {
-                zone_type* zone = (zone_type*) rrset->zone;
-                log_rrset(ldns_rr_owner(rrset->rrs[i].rr), rrset->rrtype,
-                    "error printing RRset", LOG_CRIT);
-                zone->adoutbound->error = 1;
-                break;
+        if (! skip_rrsigs) {
+            result = ODS_STATUS_OK;
+            while((rrsig = collection_iterator(rrset->rrsigs))) {
+                if (result == ODS_STATUS_OK) {
+                    result = util_rr_print(fd, rrsig->rr);
+                    if (result != ODS_STATUS_OK) {
+                        zone_type* zone = rrset->zone;
+                        log_rrset(ldns_rr_owner(rrset->rrs[i].rr), rrset->rrtype,
+                            "error printing RRset", LOG_CRIT);
+                        zone->adoutbound->error = 1;
+                    }
+                }
             }
         }
+        if (status) {
+            *status = result;
+        }
     }
-    if (status) {
-        *status = result;
-    }
-    return;
 }
 
 
@@ -852,31 +845,21 @@ void
 rrset_cleanup(rrset_type* rrset)
 {
     uint16_t i = 0;
-    zone_type* zone = NULL;
     if (!rrset) {
        return;
     }
     rrset_cleanup(rrset->next);
     rrset->next = NULL;
     rrset->domain = NULL;
-    zone = (zone_type*) rrset->zone;
     for (i=0; i < rrset->rr_count; i++) {
         ldns_rr_free(rrset->rrs[i].rr);
         rrset->rrs[i].owner = NULL;
     }
-    for (i=0; i < rrset->rrsig_count; i++) {
-        allocator_deallocate(zone->allocator,
-            (void*)rrset->rrsigs[i].key_locator);
-        ldns_rr_free(rrset->rrsigs[i].rr);
-        rrset->rrsigs[i].owner = NULL;
-    }
-    allocator_deallocate(zone->allocator, (void*) rrset->rrs);
-    allocator_deallocate(zone->allocator, (void*) rrset->rrsigs);
-    allocator_deallocate(zone->allocator, (void*) rrset);
-    return;
+    collection_destroy(&rrset->rrsigs);
+    free(rrset->rrs);
+    free(rrset);
 }
 
-
 /**
  * Backup RRset.
  *
@@ -884,20 +867,16 @@ rrset_cleanup(rrset_type* rrset)
 void
 rrset_backup2(FILE* fd, rrset_type* rrset)
 {
+    rrsig_type* rrsig;
     char* str = NULL;
-    uint16_t i = 0;
     if (!rrset || !fd) {
         return;
     }
-    for (i=0; i < rrset->rrsig_count; i++) {
-        str = ldns_rr2str(rrset->rrsigs[i].rr);
-        if (!str) {
-            continue;
+    while((rrsig = collection_iterator(rrset->rrsigs))) {
+        if ((str = ldns_rr2str(rrsig->rr))) {
+            fprintf(fd, "%.*s; {locator %s flags %u}\n", (int)strlen(str)-1, str,
+                    rrsig->key_locator, rrsig->key_flags);
+            free(str);
         }
-        str[(strlen(str))-1] = '\0';
-        fprintf(fd, "%s; {locator %s flags %u}\n", str,
-            rrset->rrsigs[i].key_locator, rrset->rrsigs[i].key_flags);
-        free((void*)str);
     }
-    return;
 }
