@@ -30,16 +30,16 @@
  */
 
 #include "adapter/adapter.h"
-#include "shared/allocator.h"
-#include "shared/file.h"
-#include "shared/hsm.h"
-#include "shared/locks.h"
-#include "shared/log.h"
-#include "shared/status.h"
-#include "shared/util.h"
+#include "file.h"
+#include "hsm.h"
+#include "locks.h"
+#include "log.h"
+#include "status.h"
+#include "util.h"
 #include "signer/backup.h"
 #include "signer/zone.h"
 #include "wire/netio.h"
+#include "compat.h"
 
 #include <ldns/ldns.h>
 
@@ -53,32 +53,30 @@ static const char* zone_str = "zone";
 zone_type*
 zone_create(char* name, ldns_rr_class klass)
 {
-    allocator_type* allocator = NULL;
     zone_type* zone = NULL;
+    int err;
 
     if (!name || !klass) {
         return NULL;
     }
-    allocator = allocator_create(malloc, free);
-    if (!allocator) {
-        ods_log_error("[%s] unable to create zone %s: allocator_create() "
-            "failed", zone_str, name);
-        return NULL;
-    }
-    zone = (zone_type*) allocator_alloc(allocator, sizeof(zone_type));
-    if (!zone) {
-        ods_log_error("[%s] unable to create zone %s: allocator_alloc()",
-            "failed", zone_str, name);
-        allocator_cleanup(allocator);
-        return NULL;
-    }
-    zone->allocator = allocator;
+    CHECKALLOC(zone = (zone_type*) calloc(1, sizeof(zone_type)));
     /* [start] PS 9218653: Drop trailing dot in domain name */
     if (strlen(name) > 1 && name[strlen(name)-1] == '.') {
         name[strlen(name)-1] = '\0';
     }
     /* [end] PS 9218653 */
-    zone->name = allocator_strdup(allocator, name);
+
+    if (pthread_mutex_init(&zone->zone_lock, NULL)) {
+        free(zone);
+        return NULL;
+    }
+    if (pthread_mutex_init(&zone->xfr_lock, NULL)) {
+        (void)pthread_mutex_destroy(&zone->zone_lock);
+        free(zone);
+        return NULL;
+    }
+
+    zone->name = strdup(name);
     if (!zone->name) {
         ods_log_error("[%s] unable to create zone %s: allocator_strdup() "
             "failed", zone_str, name);
@@ -122,12 +120,10 @@ zone_create(char* name, ldns_rr_class klass)
         return NULL;
     }
     zone->stats = stats_create();
-    lock_basic_init(&zone->zone_lock);
-    lock_basic_init(&zone->xfr_lock);
+    zone->rrstore = rrset_store_initialize();
     return zone;
 }
 
-
 /**
  * Load signer configuration for zone.
  *
@@ -164,6 +160,9 @@ zone_load_signconf(zone_type* zone, signconf_type** new_signconf)
         free((void*)datestamp);
         *new_signconf = signconf;
     } else if (status == ODS_STATUS_UNCHANGED) {
+        /* OPENDNSSEC-686: changes happening within one second will not be
+         * seen
+         */
         (void)time_datestamp(zone->signconf->last_modified,
             "%Y-%m-%d %T", &datestamp);
         ods_log_verbose("[%s] zone %s signconf file %s is unchanged since "
@@ -195,7 +194,7 @@ zone_reschedule_task(zone_type* zone, schedule_type* taskq, task_id what)
      ods_log_assert(zone->task);
      ods_log_debug("[%s] reschedule task for zone %s", zone_str, zone->name);
      lock_basic_lock(&taskq->schedule_lock);
-     task = unschedule_task(taskq, (task_type*) zone->task);
+     task = unschedule_task(taskq, zone->task);
      if (task != NULL) {
          if (task->what != what) {
              task->halted = task->what;
@@ -213,7 +212,7 @@ zone_reschedule_task(zone_type* zone, schedule_type* taskq, task_id what)
          ods_log_verbose("[%s] unable to reschedule task for zone %s now: "
              "task is not queued (task will be rescheduled when it is put "
              "back on the queue)", zone_str, zone->name);
-         task = (task_type*) zone->task;
+         task = zone->task;
          task->interrupt = what;
          /* task->halted(_when) set by worker */
      }
@@ -232,7 +231,7 @@ zone_publish_dnskeys(zone_type* zone)
 {
     hsm_ctx_t* ctx = NULL;
     uint32_t ttl = 0;
-    uint16_t i = 0;
+    unsigned int i;
     ods_status status = ODS_STATUS_OK;
     rrset_type* rrset = NULL;
     rr_type* dnskey = NULL;
@@ -249,8 +248,8 @@ zone_publish_dnskeys(zone_type* zone)
             "error creating libhsm context", zone_str, zone->name);
         return ODS_STATUS_HSM_ERR;
     }
-    /* dnskey ttl */
     ttl = zone->default_ttl;
+    /* dnskey ttl */
     if (zone->signconf->dnskey_ttl) {
         ttl = (uint32_t) duration2time(zone->signconf->dnskey_ttl);
     }
@@ -261,14 +260,25 @@ zone_publish_dnskeys(zone_type* zone)
         }
         if (!zone->signconf->keys->keys[i].dnskey) {
             /* get dnskey */
-            status = lhsm_get_key(ctx, zone->apex,
-                &zone->signconf->keys->keys[i]);
-            if (status != ODS_STATUS_OK) {
-                ods_log_error("[%s] unable to publish dnskeys for zone %s: "
-                    "error creating dnskey", zone_str, zone->name);
-                break;
+            if  (zone->signconf->keys->keys[i].resourcerecord) {
+                if ((status = rrset_getliteralrr(&zone->signconf->keys->keys[i].dnskey, zone->signconf->keys->keys[i].resourcerecord, ttl, zone->apex)) != ODS_STATUS_OK) {
+                    ods_log_error("[%s] unable to publish dnskeys for zone %s: "
+                            "error decoding literal dnskey", zone_str, zone->name);
+		    hsm_destroy_context(ctx);
+                    return status;
+                }
+            } else {
+                status = lhsm_get_key(ctx, zone->apex,
+                        &zone->signconf->keys->keys[i]);
+                if (status != ODS_STATUS_OK) {
+                    ods_log_error("[%s] unable to publish dnskeys for zone %s: "
+                            "error creating dnskey", zone_str, zone->name);
+                    break;
+                }
             }
         }
+        ods_log_debug("[%s] publish %s DNSKEY locator %s", zone_str,
+            zone->name, zone->signconf->keys->keys[i].locator);
         ods_log_assert(zone->signconf->keys->keys[i].dnskey);
         ldns_rr_set_ttl(zone->signconf->keys->keys[i].dnskey, ttl);
         ldns_rr_set_class(zone->signconf->keys->keys[i].dnskey, zone->klass);
@@ -322,8 +332,6 @@ zone_rollback_dnskeys(zone_type* zone)
             }
         }
     }
-    /* done */
-    return;
 }
 
 
@@ -420,7 +428,6 @@ zone_rollback_nsec3param(zone_type* zone)
             zone->signconf->nsec3params->rr = NULL;
         }
     }
-    return;
 }
 
 
@@ -448,6 +455,8 @@ zone_prepare_keys(zone_type* zone)
     }
     /* prepare keys */
     for (i=0; i < zone->signconf->keys->count; i++) {
+        if(zone->signconf->dnskey_signature != NULL && zone->signconf->keys->keys[i].ksk)
+            continue;
         /* get dnskey */
         status = lhsm_get_key(ctx, zone->apex, &zone->signconf->keys->keys[i]);
         if (status != ODS_STATUS_OK) {
@@ -609,6 +618,7 @@ zone_add_rr(zone_type* zone, ldns_rr* rr, int do_stats)
         domain_add_rrset(domain, rrset);
     }
     record = rrset_lookup_rr(rrset, rr);
+
     if (record && ldns_rr_ttl(rr) != ldns_rr_ttl(record->rr))
         record = NULL;
 
@@ -702,7 +712,7 @@ zone_del_nsec3params(zone_type* zone)
 
     domain = namedb_lookup_domain(zone->db, zone->apex);
     if (!domain) {
-        ods_log_warning("[%s] unable to delete RR from zone %s: "
+        ods_log_verbose("[%s] unable to delete RR from zone %s: "
             "domain not found", zone_str, zone->name);
         return ODS_STATUS_UNCHANGED;
     }
@@ -786,7 +796,6 @@ zone_merge(zone_type* z1, zone_type* z2)
         z1->adoutbound = adtmp;
         adtmp = NULL;
     }
-    return;
 }
 
 
@@ -797,15 +806,9 @@ zone_merge(zone_type* z1, zone_type* z2)
 void
 zone_cleanup(zone_type* zone)
 {
-    allocator_type* allocator;
-    lock_basic_type zone_lock;
-    lock_basic_type xfr_lock;
     if (!zone) {
         return;
     }
-    allocator = zone->allocator;
-    zone_lock = zone->zone_lock;
-    xfr_lock = zone->xfr_lock;
     ldns_rdf_deep_free(zone->apex);
     adapter_cleanup(zone->adinbound);
     adapter_cleanup(zone->adoutbound);
@@ -815,16 +818,15 @@ zone_cleanup(zone_type* zone)
     notify_cleanup(zone->notify);
     signconf_cleanup(zone->signconf);
     stats_cleanup(zone->stats);
-    allocator_deallocate(allocator, (void*) zone->notify_command);
-    allocator_deallocate(allocator, (void*) zone->notify_args);
-    allocator_deallocate(allocator, (void*) zone->policy_name);
-    allocator_deallocate(allocator, (void*) zone->signconf_filename);
-    allocator_deallocate(allocator, (void*) zone->name);
-    allocator_deallocate(allocator, (void*) zone);
-    allocator_cleanup(allocator);
-    lock_basic_destroy(&xfr_lock);
-    lock_basic_destroy(&zone_lock);
-    return;
+    free(zone->notify_command);
+    free(zone->notify_args);
+    free((void*)zone->policy_name);
+    free((void*)zone->signconf_filename);
+    free((void*)zone->name);
+    collection_class_destroy(&zone->rrstore);
+    lock_basic_destroy(&zone->xfr_lock);
+    lock_basic_destroy(&zone->zone_lock);
+    free(zone);
 }
 
 
@@ -914,6 +916,8 @@ zone_recover2(zone_type* zone)
             !backup_read_duration(fd, &zone->signconf->sig_validity_default) |
             !backup_read_check_str(fd, "denial") |
             !backup_read_duration(fd,&zone->signconf->sig_validity_denial) |
+            !backup_read_check_str(fd, "keyset") |
+            !backup_read_duration(fd,&zone->signconf->sig_validity_keyset) |
             !backup_read_check_str(fd, "jitter") |
             !backup_read_duration(fd, &zone->signconf->sig_jitter) |
             !backup_read_check_str(fd, "offset") |
@@ -947,8 +951,7 @@ zone_recover2(zone_type* zone)
                     "nsec3parameters error", zone_str, zone->name);
                 goto recover_error2;
             }
-            zone->signconf->nsec3_salt = allocator_strdup(
-                zone->signconf->allocator, salt);
+            zone->signconf->nsec3_salt = strdup(salt);
             free((void*) salt);
             salt = NULL;
             zone->signconf->nsec3params = nsec3params_create(
@@ -995,7 +998,8 @@ zone_recover2(zone_type* zone)
             goto recover_error2;
         }
         /* publish nsec3param */
-        status = zone_publish_nsec3param(zone);
+        if (!zone->signconf->passthrough)
+            status = zone_publish_nsec3param(zone);
         if (status != ODS_STATUS_OK) {
             ods_log_error("[%s] corrupted backup file zone %s: unable to "
                 "publish nsec3param (%s)", zone_str, zone->name,
@@ -1054,6 +1058,7 @@ zone_recover2(zone_type* zone)
         }
         return ODS_STATUS_OK;
     }
+    free(filename);
     return ODS_STATUS_UNCHANGED;
 
 recover_error2:
@@ -1109,7 +1114,7 @@ zone_backup2(zone_type* zone)
     fd = ods_fopen(tmpfile, NULL, "w");
     if (fd) {
         fprintf(fd, "%s\n", ODS_SE_FILE_MAGIC_V3);
-        task = (task_type*) zone->task;
+        task = zone->task;
         fprintf(fd, ";;Time: %u\n", (unsigned) task->when);
         /** Backup zone */
         fprintf(fd, ";;Zone: name %s class %i inbound %u internal %u "
