@@ -24,29 +24,24 @@
  *
  */
 
-/**
- *
- * The zonelist and all.
- */
-
 #ifndef SIGNER_ZONELIST_H
 #define SIGNER_ZONELIST_H
 
-#include "shared/allocator.h"
-#include "shared/locks.h"
-#include "signer/zone.h"
-
 #include <ldns/ldns.h>
 #include <stdio.h>
 #include <time.h>
 
+typedef struct zonelist_struct zonelist_type;
+
+#include "status.h"
+#include "locks.h"
+#include "signer/zone.h"
+
 /**
  * Zone list
  *
  */
-typedef struct zonelist_struct zonelist_type;
 struct zonelist_struct {
-    allocator_type* allocator;
     ldns_rbtree_t* zones;
     time_t last_modified;
     int just_added;
@@ -61,7 +56,7 @@ struct zonelist_struct {
  * \return zonelist_type* created zone list
  *
  */
-zonelist_type* zonelist_create(allocator_type* allocator);
+zonelist_type* zonelist_create(void);
 
 /**
  * Lookup zone by name and class.
