@@ -30,7 +30,8 @@
  */
 
 #include "config.h"
-#include "shared/log.h"
+#include <stdlib.h>
+#include "log.h"
 #include "wire/listener.h"
 
 static const char* listener_str = "listener";
@@ -41,20 +42,10 @@ static const char* listener_str = "listener";
  *
  */
 listener_type*
-listener_create(allocator_type* allocator)
+listener_create()
 {
     listener_type* listener = NULL;
-    if (!allocator) {
-        return NULL;
-    }
-    listener = (listener_type*) allocator_alloc(allocator,
-        sizeof(listener_type));
-    if (!listener) {
-        ods_log_error("[%s] create listener failed: allocator_alloc() failed",
-            listener_str);
-        return NULL;
-    }
-    listener->allocator = allocator;
+    CHECKALLOC(listener = (listener_type*) malloc(sizeof(listener_type)));
     listener->count = 0;
     listener->interfaces = NULL;
     return listener;
@@ -72,25 +63,18 @@ listener_push(listener_type* listener, char* address, int family, char* port)
     ods_log_assert(listener);
     ods_log_assert(address);
     ifs_old = listener->interfaces;
-    listener->interfaces = (interface_type*) allocator_alloc(
-        listener->allocator, (listener->count + 1) * sizeof(interface_type));
-    if (!listener->interfaces) {
-        ods_fatal_exit("[%s] fatal unable to add interface: allocator_alloc() failed",
-            listener_str);
-    }
+    CHECKALLOC(listener->interfaces = (interface_type*) malloc((listener->count + 1) * sizeof(interface_type)));
     if (ifs_old) {
         memcpy(listener->interfaces, ifs_old,
            (listener->count) * sizeof(interface_type));
     }
-    allocator_deallocate(listener->allocator, (void*) ifs_old);
+    free(ifs_old);
     listener->count++;
-    listener->interfaces[listener->count -1].address =
-        allocator_strdup(listener->allocator, address);
+    listener->interfaces[listener->count -1].address = strdup(address);
     listener->interfaces[listener->count -1].family = family;
 
     if (port) {
-        listener->interfaces[listener->count -1].port =
-            allocator_strdup(listener->allocator, port);
+        listener->interfaces[listener->count -1].port = strdup(port);
     } else{
         listener->interfaces[listener->count -1].port = NULL;
     }
@@ -101,7 +85,7 @@ listener_push(listener_type* listener, char* address, int family, char* port)
         if (inet_pton(listener->interfaces[listener->count -1].family,
             listener->interfaces[listener->count -1].address,
             &listener->interfaces[listener->count -1].addr.addr6) != 1) {
-            ods_log_error("[%s] bad ip address '%s'",
+            ods_log_error("[%s] bad ip address '%s'", listener_str,
                 listener->interfaces[listener->count -1].address);
             return NULL;
         }
@@ -110,7 +94,7 @@ listener_push(listener_type* listener, char* address, int family, char* port)
         if (inet_pton(listener->interfaces[listener->count -1].family,
             listener->interfaces[listener->count -1].address,
             &listener->interfaces[listener->count -1].addr.addr) != 1) {
-            ods_log_error("[%s] bad ip address '%s'",
+            ods_log_error("[%s] bad ip address '%s'", listener_str,
                 listener->interfaces[listener->count -1].address);
             return NULL;
         }
@@ -119,86 +103,6 @@ listener_push(listener_type* listener, char* address, int family, char* port)
 }
 
 
-/**
- * Print interface.
- *
- */
-static void
-interface_print(FILE* fd, interface_type* i)
-{
-    if (!fd || !i) {
-        return;
-    }
-    fprintf(fd, "<Interface>");
-    if (i->family == AF_INET && i->address) {
-        fprintf(fd, "<IPv4>%s</IPv4>", i->address);
-    } else if (i->family == AF_INET6 && i->address) {
-        fprintf(fd, "<IPv6>%s</IPv6>", i->address);
-    }
-    if (i->port) {
-        fprintf(fd, "<Port>%s</Port>", i->port);
-    }
-    fprintf(fd, "</Interface>\n");
-    return;
-}
-
-
-/**
- * Print listener.
- *
- */
-void
-listener_print(FILE* fd, listener_type* listener)
-{
-    uint16_t i = 0;
-    if (!fd || !listener || listener->count <= 0) {
-        return;
-    }
-    fprintf(fd, "<Listener>\n");
-    for (i=0; i < listener->count; i++) {
-        interface_print(fd, &listener->interfaces[i]);
-    }
-    fprintf(fd, "</Listener>\n");
-    return;
-}
-
-
-/**
- * Log interface.
- *
- */
-static void
-interface_log(interface_type* i)
-{
-    if (!i) {
-        return;
-    }
-    ods_log_debug("[%s] FAMILY[%s] ADDRESS[%s] PORT[%s]", listener_str,
-        i->family==AF_INET6?"IPv6":"IPv4",
-        i->address?i->address:"localhost",
-        i->port?i->port:DNS_PORT_STRING);
-    return;
-}
-
-
-/**
- * Log listener.
- *
- */
-void
-listener_log(listener_type* listener)
-{
-    uint16_t i = 0;
-    if (!listener || listener->count <= 0) {
-        return;
-    }
-    for (i=0; i < listener->count; i++) {
-        interface_log(&listener->interfaces[i]);
-    }
-    return;
-}
-
-
 /**
  * Clean up interface.
  *
@@ -211,7 +115,6 @@ interface_cleanup(interface_type* i)
     }
     free((void*)i->port);
     free((void*)i->address);
-    return;
 }
 
 
@@ -223,15 +126,12 @@ void
 listener_cleanup(listener_type* listener)
 {
     uint16_t i = 0;
-    allocator_type* allocator = NULL;
     if (!listener) {
         return;
     }
     for (i=0; i < listener->count; i++) {
         interface_cleanup(&listener->interfaces[i]);
     }
-    allocator = listener->allocator;
-    allocator_deallocate(allocator, (void*) listener->interfaces);
-    allocator_deallocate(allocator, (void*) listener);
-    return;
+    free(listener->interfaces);
+    free(listener);
 }
