@@ -32,8 +32,8 @@
 #include "config.h"
 #include "daemon/dnshandler.h"
 #include "daemon/engine.h"
-#include "shared/file.h"
-#include "shared/util.h"
+#include "file.h"
+#include "util.h"
 #include "wire/axfr.h"
 #include "wire/query.h"
 
@@ -47,32 +47,22 @@ const char* query_str = "query";
 query_type*
 query_create(void)
 {
-    allocator_type* allocator = NULL;
     query_type* q = NULL;
-    allocator = allocator_create(malloc, free);
-    if (!allocator) {
-        return NULL;
-    }
-    q = (query_type*) allocator_alloc(allocator, sizeof(query_type));
-    if (!q) {
-        allocator_cleanup(allocator);
-        return NULL;
-    }
-    q->allocator = allocator;
+    CHECKALLOC(q = (query_type*) malloc(sizeof(query_type)));
     q->buffer = NULL;
     q->tsig_rr = NULL;
     q->axfr_fd = NULL;
-    q->buffer = buffer_create(allocator, PACKET_BUFFER_SIZE);
+    q->buffer = buffer_create(PACKET_BUFFER_SIZE);
     if (!q->buffer) {
         query_cleanup(q);
         return NULL;
     }
-    q->tsig_rr = tsig_rr_create(allocator);
+    q->tsig_rr = tsig_rr_create();
     if (!q->tsig_rr) {
         query_cleanup(q);
         return NULL;
     }
-    q->edns_rr = edns_rr_create(allocator);
+    q->edns_rr = edns_rr_create();
     if (!q->edns_rr) {
         query_cleanup(q);
         return NULL;
@@ -112,7 +102,6 @@ query_reset(query_type* q, size_t maxlen, int is_tcp)
     }
     q->serial = 0;
     q->startpos = 0;
-    return;
 }
 
 
@@ -273,19 +262,18 @@ query_parse_soa(buffer_type* buffer, uint32_t* serial)
  * prepare notify reply packet in q->buffer.
  */
 static query_state
-query_process_notify(query_type* q, ldns_rr_type qtype, void* engine)
+query_process_notify(query_type* q, ldns_rr_type qtype, engine_type* engine)
 {
-    engine_type* e = (engine_type*) engine;
     dnsin_type* dnsin = NULL;
     uint16_t count = 0;
     uint16_t rrcount = 0;
     uint32_t serial = 0;
     size_t pos = 0;
     char address[128];
-    if (!e || !q || !q->zone) {
+    if (!engine || !q || !q->zone) {
         return QUERY_DISCARDED;
     }
-    ods_log_assert(e->dnshandler);
+    ods_log_assert(engine->dnshandler);
     ods_log_assert(q->zone->name);
     ods_log_verbose("[%s] incoming notify for zone %s", query_str,
         q->zone->name);
@@ -392,10 +380,24 @@ query_process_notify(query_type* q, ldns_rr_type qtype, void* engine)
                     q->zone->name);
             }
             xfrd_set_timer_now(q->zone->xfrd);
-            dnshandler_fwd_notify(e->dnshandler, buffer_begin(q->buffer),
+            dnshandler_fwd_notify(engine->dnshandler, buffer_begin(q->buffer),
                 buffer_remaining(q->buffer));
         }
+    } else { /* Empty answer section, no SOA. We still need to process
+        the notify according to the RFC */
+        /* forward notify to xfrd */
+        if (addr2ip(q->addr, address, sizeof(address))) {
+            ods_log_verbose("[%s] forward notify for zone %s from client %s",
+                query_str, q->zone->name, address);
+        } else {
+            ods_log_verbose("[%s] forward notify for zone %s", query_str,
+                q->zone->name);
+        }
+        xfrd_set_timer_now(q->zone->xfrd);
+        dnshandler_fwd_notify(engine->dnshandler, buffer_begin(q->buffer),
+            buffer_remaining(q->buffer));
     }
+
     /* send notify ok */
     buffer_pkt_set_qr(q->buffer);
     buffer_pkt_set_aa(q->buffer);
@@ -502,6 +504,7 @@ response_encode_rr(query_type* q, ldns_rr* rr, ldns_pkt_section section)
 static uint16_t
 response_encode_rrset(query_type* q, rrset_type* rrset, ldns_pkt_section section)
 {
+    rrsig_type* rrsig;
     uint16_t i = 0;
     uint16_t added = 0;
     ods_log_assert(q);
@@ -512,8 +515,8 @@ response_encode_rrset(query_type* q, rrset_type* rrset, ldns_pkt_section section
         added += response_encode_rr(q, rrset->rrs[i].rr, section);
     }
     if (q->edns_rr && q->edns_rr->dnssec_ok) {
-        for (i = 0; i < rrset->rrsig_count; i++) {
-            added += response_encode_rr(q, rrset->rrsigs[i].rr, section);
+        while((rrsig = collection_iterator(rrset->rrsigs))) {
+            added += response_encode_rr(q, rrsig->rr, section);
         }
     }
     /* truncation? */
@@ -548,7 +551,6 @@ response_encode(query_type* q, response_type* r)
     buffer_pkt_set_arcount(q->buffer, counts[LDNS_SECTION_ADDITIONAL]);
     buffer_pkt_set_qr(q->buffer);
     buffer_pkt_set_aa(q->buffer);
-    return;
 }
 
 
@@ -621,7 +623,6 @@ query_prepare(query_type* q)
     buffer_set_limit(q->buffer, buffer_capacity(q->buffer));
     q->reserved_space = edns_rr_reserved_space(q->edns_rr);
     q->reserved_space += tsig_rr_reserved_space(q->tsig_rr);
-    return;
 }
 
 
@@ -836,7 +837,7 @@ query_find_tsig(query_type* q)
  *
  */
 query_state
-query_process(query_type* q, void* engine)
+query_process(query_type* q, engine_type* engine)
 {
     ldns_status status = LDNS_STATUS_OK;
     ldns_pkt* pkt = NULL;
@@ -844,11 +845,10 @@ query_process(query_type* q, void* engine)
     ldns_pkt_rcode rcode = LDNS_RCODE_NOERROR;
     ldns_pkt_opcode opcode = LDNS_PACKET_QUERY;
     ldns_rr_type qtype = LDNS_RR_TYPE_SOA;
-    engine_type* e = (engine_type*) engine;
-    ods_log_assert(e);
+    ods_log_assert(engine);
     ods_log_assert(q);
     ods_log_assert(q->buffer);
-    if (!e || !q || !q->buffer) {
+    if (!engine || !q || !q->buffer) {
         ods_log_error("[%s] drop query: assertion error", query_str);
         return QUERY_DISCARDED; /* should not happen */
     }
@@ -869,10 +869,14 @@ query_process(query_type* q, void* engine)
         return query_formerr(q);
     }
     rr = ldns_rr_list_rr(ldns_pkt_question(pkt), 0);
-    lock_basic_lock(&e->zonelist->zl_lock);
+    if (!rr) {
+        ods_log_debug("[%s] no RRset in query section, ignoring", query_str);
+        return QUERY_DISCARDED; /* no RRset in query */
+    }
+    lock_basic_lock(&engine->zonelist->zl_lock);
     /* we can just lookup the zone, because we will only handle SOA queries,
        zone transfers, updates and notifies */
-    q->zone = zonelist_lookup_zone_by_dname(e->zonelist, ldns_rr_owner(rr),
+    q->zone = zonelist_lookup_zone_by_dname(engine->zonelist, ldns_rr_owner(rr),
         ldns_rr_get_class(rr));
     /* don't answer for zones that are just added */
     if (q->zone && q->zone->zl_status == ZONE_ZL_ADDED) {
@@ -881,7 +885,7 @@ query_process(query_type* q, void* engine)
             query_str, q->zone->name);
         q->zone = NULL;
     }
-    lock_basic_unlock(&e->zonelist->zl_lock);
+    lock_basic_unlock(&engine->zonelist->zl_lock);
     if (!q->zone) {
         ods_log_debug("[%s] zone not found", query_str);
         return query_servfail(q);
@@ -951,16 +955,15 @@ query_overflow(query_type* q)
  *
  */
 void
-query_add_optional(query_type* q, void* engine)
+query_add_optional(query_type* q, engine_type* engine)
 {
-    engine_type* e = (engine_type*) engine;
     edns_data_type* edns = NULL;
-    if (!q || !e) {
+    if (!q || !engine) {
         return;
     }
     /** First EDNS */
     if (q->edns_rr) {
-        edns = &e->edns;
+        edns = &engine->edns;
         switch (q->edns_rr->status) {
             case EDNS_NOT_PRESENT:
                 break;
@@ -1023,7 +1026,6 @@ query_add_optional(query_type* q, void* engine)
              }
         }
     }
-    return;
 }
 
 
@@ -1091,18 +1093,15 @@ query_add_rr_tc:
 void
 query_cleanup(query_type* q)
 {
-    allocator_type* allocator = NULL;
     if (!q) {
         return;
     }
-    allocator = q->allocator;
     if (q->axfr_fd) {
         ods_fclose(q->axfr_fd);
         q->axfr_fd = NULL;
     }
-    buffer_cleanup(q->buffer, allocator);
+    buffer_cleanup(q->buffer);
     tsig_rr_cleanup(q->tsig_rr);
-    allocator_deallocate(allocator, (void*)q);
-    allocator_cleanup(allocator);
-    return;
+    edns_rr_cleanup(q->edns_rr);
+    free(q);
 }
